﻿/* ============================================================
* Author:       liubingsheng
* Time:         2016/11/29 14:52:25
* FileName:     RedisSystem
* Purpose:      Redis
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;
using Config;
using System.Data;
using Log;
using StackExchange.Redis;
using System.IO;
using ProtoBuf;
using MongoDB.Bson;
using MongoDB.Driver;
using MongoDB.Bson.IO;
using MongoDB.Driver.Core.Servers;
using System.Linq.Expressions;
using MongoDB.Bson.Serialization.Conventions;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Options;
using AdventureClient;
using Protocols;

namespace ServerLib {
    class DictionaryRepresentationConvention : ConventionBase, IMemberMapConvention {
        private readonly DictionaryRepresentation _dictionaryRepresentation;
        public DictionaryRepresentationConvention(DictionaryRepresentation dictionaryRepresentation) {
            _dictionaryRepresentation = dictionaryRepresentation;
        }
        public void Apply(BsonMemberMap memberMap) {
            memberMap.SetSerializer(ConfigureSerializer(memberMap.GetSerializer()));
        }
        private IBsonSerializer ConfigureSerializer(IBsonSerializer serializer) {
            var dictionaryRepresentationConfigurable = serializer as IDictionaryRepresentationConfigurable;
            if (dictionaryRepresentationConfigurable != null) {
                serializer = dictionaryRepresentationConfigurable.WithDictionaryRepresentation(_dictionaryRepresentation);
            }

            var childSerializerConfigurable = serializer as IChildSerializerConfigurable;
            return childSerializerConfigurable == null
                ? serializer
                : childSerializerConfigurable.WithChildSerializer(ConfigureSerializer(childSerializerConfigurable.ChildSerializer));
        }
    }

    public class MongoSystem {
        public MongoSystem() {
        }

        private string ip = "";
        private string port = "";

        protected MongoClient client;

        /// <summary>
        /// 内部初始化
        /// </summary>
        public bool Init() {
            this.ip = "127.0.0.1";
            this.port = "27017";
            string connString = "mongodb://" + ip + ":" + port;
            client = new MongoClient(connString);

            ConventionRegistry.Register(
            "DictionaryRepresentationConvention",
            new ConventionPack { new DictionaryRepresentationConvention(DictionaryRepresentation.ArrayOfArrays) }, _ => true);

            BsonSerializer.RegisterSerializer(typeof(DateTime), new MongoDB.Bson.Serialization.Serializers.DateTimeSerializer(DateTimeKind.Local));

            return true;
        }


        /// <summary>
        /// 默认数据库
        /// </summary>
        public IMongoDatabase db {
            get {
                var db = client.GetDatabase("gamedb");
                return db;
            }
        }

        /// <summary>
        /// 添加数据到集合中
        /// </summary>
        /// <returns></returns>
        public void AddInfoToCollection<T>(T t, string key) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            col.InsertOne(t);
        }

        /// <summary>
        /// 根据key获得集合
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        public List<T> GetInfoCollectionByKey<T>(string key) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            List<T> list = col.Find(new BsonDocument()).ToList();
            return list;
        }

        /// <summary>
        /// 获得集合的长度
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        public long GetCollectionCount<T>(string key) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            var count = col.CountDocuments(new BsonDocument());
            return count;
        }

        /// <summary>
        /// 查找对象通过一个属性值
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="Id"></param>
        /// <param name="ShuXing">搜索的属性</param>
        /// <returns></returns>
        public T GetTFromCollectionByFilter<T>(string key, FilterDefinition<T> filter) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            var collection = col.Find(filter).ToList().FirstOrDefault();
            return collection;
        }

        /// <summary>
        /// 根据某一个属性查找
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="filter"></param>
        /// <returns></returns>
        public async Task<T> GetTFromCollectionByFilterAsync<T>(string key, FilterDefinition<T> filter) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            var collection = (await col.Find(filter).ToListAsync()).FirstOrDefault();
            return collection;
        }

        /// <summary>
        /// 更新一条数据通过过滤器和更新器
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="filter"></param>
        /// <param name="update"></param>
        public void UpdateTFromCollectionByFiterAndUpdate<T>(string key, FilterDefinition<T> filter, UpdateDefinition<T> update) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            col.UpdateOne(filter, update);
        }

        /// <summary>
        /// 删除一条数据通过过滤器
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="filter"></param>
        public void DelTFromCollectionByFilter<T>(string key, FilterDefinition<T> filter) {
            var col = Env.mongodb.db.GetCollection<T>(key);
            col.DeleteOne(filter);
        }
    }
}
